<?php

namespace Drupal\commerce_registration\Access;

use Drupal\commerce_product\Entity\ProductInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatch;
use Drupal\Core\Session\AccountInterface;
use Drupal\registration\RegistrationManagerInterface;

/**
 * Checks access for the Manage Commerce Registrations route.
 */
class ManageCommerceRegistrationsAccessCheck implements AccessInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The registration manager.
   *
   * @var \Drupal\registration\RegistrationManagerInterface
   */
  protected RegistrationManagerInterface $registrationManager;

  /**
   * ManageCommerceRegistrationsAccessCheck constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\registration\RegistrationManagerInterface $registration_manager
   *   The registration manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RegistrationManagerInterface $registration_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->registrationManager = $registration_manager;
  }

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   * @param \Drupal\Core\Routing\RouteMatch $route_match
   *   Run access checks for this route.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   */
  public function access(AccountInterface $account, RouteMatch $route_match): AccessResultInterface {
    // Allow access if the product in the route has at least one variation with
    // its registration type field set, and the given user account has
    // permissions to administer or update registrations.
    $product = $this->registrationManager->getEntityFromParameters($route_match->getParameters());
    if ($product instanceof ProductInterface) {
      $handler = $this->entityTypeManager->getHandler('registration', 'host_entity');
      foreach ($product->getVariations() as $variation) {
        $host_entity = $handler->createHostEntity($variation);
        if ($type = $host_entity->getRegistrationTypeBundle()) {
          $view_any = $account->hasPermission("view any registration");
          $access =
               $account->hasPermission("administer registration")
            || ($account->hasPermission("administer $type registration") && $view_any)
            || ($account->hasPermission("administer own $type registration") && $variation->access('update', $account) && $view_any);
          $access_result = AccessResult::allowedIf($access)
            // Recalculate this result if the relevant entities are updated.
            ->cachePerPermissions()
            ->addCacheableDependency($variation)
            ->addCacheableDependency($product);
          if ($access_result->isAllowed()) {
            return $access_result;
          }
        }
      }
    }

    // No product available, or its registration fields are set to disable
    // registrations. Return neutral so other modules can have a say in
    // whether registration is allowed. Most likely no other module will
    // allow the registration, so this will disable the route. This would
    // in turn hide the Manage Registrations tab for the product.
    $access_result = AccessResult::neutral();

    // Recalculate this result if the relevant entities are updated.
    $access_result->cachePerPermissions();
    if ($product instanceof ProductInterface) {
      $access_result->addCacheableDependency($product);
    }
    return $access_result;
  }

}
