<?php

namespace Drupal\commerce_registration\EventSubscriber;

use Drupal\commerce_order\Event\OrderEvent;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Copies the order's profiles to the customer's address book.
 */
class OrderSubscriber implements EventSubscriberInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a new OrderSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Marks held registrations as pending for submitted orders.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The event.
   */
  public function onOrderPlace(WorkflowTransitionEvent $event) {
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $event->getEntity();
    $registrations = $this->entityTypeManager->getStorage('registration')->loadByProperties([
      'order_id' => $order->id(),
    ]);
    /** @var \Drupal\registration\Entity\RegistrationInterface $registration */
    foreach ($registrations as $registration) {
      if ($registration->isHeld()) {
        $registration->set('state', 'pending');
        $registration->save();
      }
    }
  }

  /**
   * Marks active registrations as complete for fully paid orders.
   *
   * @param \Drupal\commerce_order\Event\OrderEvent $event
   *   The event.
   */
  public function onOrderPaid(OrderEvent $event) {
    $order = $event->getOrder();
    $registrations = $this->entityTypeManager->getStorage('registration')->loadByProperties([
      'order_id' => $order->id(),
    ]);
    /** @var \Drupal\registration\Entity\RegistrationInterface $registration */
    foreach ($registrations as $registration) {
      // Only mark active registrations, in case any were canceled.
      if ($registration->isActive()) {
        $registration->set('state', 'complete');
        $registration->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      'commerce_order.place.post_transition' => ['onOrderPlace', 100],
      'commerce_order.order.paid' => ['onOrderPaid', 100],
    ];
  }

}
