<?php

namespace Drupal\commerce_registration\Plugin\views\filter;

use Drupal\commerce_registration\CommerceRegistrationManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\filter\InOperator;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a filter for product variation.
 *
 * Allow selection from a list of registration enabled product variations for a
 * given product. The product must be passed as the first argument to the view
 * using this filter.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("commerce_registration_product_variation")
 */
class ProductVariation extends InOperator {

  /**
   * The commerce registration manager.
   *
   * @var \Drupal\commerce_registration\CommerceRegistrationManagerInterface
   */
  protected CommerceRegistrationManagerInterface $commerceRegistrationManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): ProductVariation {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->commerceRegistrationManager = $container->get('commerce_registration.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    if (!empty($this->value) && !empty($this->value[0])) {
      $this->ensureMyTable();

      $this->query->addWhere($this->options['group'], "$this->tableAlias.entity_id", $this->value[0], '=');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildExposedForm(&$form, FormStateInterface $form_state) {
    parent::buildExposedForm($form, $form_state);
    if (!empty($this->options['expose']['identifier'])) {
      $identifier = $this->options['expose']['identifier'];
      $form[$identifier]['#title'] = $this->t('Variation');
    }
  }

  /**
   * Build the value form when the filter is exposed.
   */
  protected function valueForm(&$form, FormStateInterface $form_state) {
    // Default to a hidden blank value so configuration schema is satisfied.
    $form['value'] = [
      '#type' => 'hidden',
      '#value' => [],
    ];

    // Create a value form when the filter is exposed.
    $exposed = $form_state->get('exposed');
    if ($exposed && !empty($this->view->args)) {
      // The view argument is assumed to be a list of product variation IDs.
      // Retrieve the associated product.
      if ($product_id = $this->commerceRegistrationManager->getProductIdFromArgument($this->view->args[0])) {
        $form['value'] = [
          '#type' => 'commerce_registration_entity_select',
          '#target_type' => 'commerce_product_variation',
          '#selection_handler' => 'commerce_registration_variation',
          '#selection_settings' => [
            'product_id' => $product_id,
          ],
          '#autocomplete_threshold' => 20,
          '#autocomplete_size' => 40,
        ];
      }
    }
  }

}
